/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MULTIPOPULATION__
#include "multipopulation.h"
#endif

#ifndef __MATHUTILS__
#include "mathutils.h"
#endif

#include <iostream>
#include <cstdlib>
#include <ctime>

using namespace std;


typedef void (*PrintProc)(const Genome *g);

/*
enum {
	kKnapsackItems = 16 // number of candidate items for knapsack
};


static unsigned kSizes[kKnapsackItems] = {
	3,
	3,
	3,
	4,

	4,
	4,
	5,
	5,

	6,
	6,
	6,
	7,

	7,
	7,
	8,
	9
};

static unsigned kValues[kKnapsackItems] = {
	2,
	2,
	4,
	4,
	
	4,
	5,
	4,
	5,
	
	5,
	6,
	6,
	7,
	
	8,
	10,
	12,
	14
};
const double kBestKnapsackFitness = 14.0;

const unsigned long kKnapsackMaxSize = 10;
*/
enum {
	kKnapsackItems = 100 // number of candidate items for knapsack
};

const double kBestKnapsackFitness = 181.0;  // best seen is 182
const unsigned long kKnapsackMaxSize = 100; // average size of 20 times 5

static unsigned kSizes[kKnapsackItems] = {
	31,
	9,
	8,
	71,
	35,
	14,
	31,
	34,
	25,
	15,
	22,
	57,
	15,
	5,
	15,
	15,
	19,
	25,
	70,
	42,
	52,
	50,
	30,
	1,
	1,
	6,
	23,
	8,
	33,
	32,
	1,
	24,
	27,
	32,
	1,
	23,
	1,
	16,
	13,
	28,
	31,
	29,
	1,
	51,
	2,
	1,
	5,
	15,
	12,
	23,
	1,
	1,
	54,
	36,
	30,
	35,
	1,
	19,
	16,
	53,
	33,
	19,
	10,
	26,
	1,
	49,
	34,
	49,
	1,
	40,
	11,
	9,
	47,
	13,
	26,
	20,
	5,
	45,
	30,
	1,
	5,
	20,
	21,
	58,
	1,
	42,
	5,
	18,
	26,
	45,
	10,
	11,
	18,
	47,
	20,
	1,
	7,
	51,
	30,
	1
};


static unsigned kValues[kKnapsackItems] = {
	11,
	9,
	4,
	14,
	3,
	1,
	16,
	6,
	1,
	4,
	1,
	8,
	1,
	2,
	3,
	9,
	1,
	3,
	7,
	13,
	8,
	1,
	3,
	11,
	10,
	5,
	2,
	5,
	1,
	6,
	7,
	10,
	1,
	8,
	7,
	3,
	7,
	4,
	1,
	1,
	7,
	2,
	8,
	9,
	12,
	8,
	2,
	3,
	7,
	7,
	3,
	4,
	3,
	5,
	4,
	1,
	9,
	10,
	1,
	5,
	1,
	4,
	1,
	10,
	1,
	3,
	10,
	17,
	1,
	7,
	8,
	5,
	7,
	6,
	2,
	1,
	5,
	5,
	6,
	1,
	4,
	1,
	3,
	8,
	9,
	1,
	1,
	3,
	12,
	13,
	1,
	1,
	13,
	11,
	6,
	6,
	12,
	9,
	1,
	10
};

// return fitness of this specimen
static double KnapsackFitness( unsigned inNumGenes, const Gene *inGene, void *inFitnessData)
{
	long totalSize=0,totalValue=0;
	for(unsigned i=0;i<inNumGenes;i++)
	{
		if( inGene[i].GetInteger() )
		{
			totalSize += kSizes[i];
			totalValue += kValues[i];
		}
	}

	if( totalSize <= kKnapsackMaxSize )
		return totalValue;
	else
		return 0;
}


static GenomeBlueprint *KnapsackBlueprint(void)
{
	GenomeBlueprint *blueprint = new GenomeBlueprint;

	if( blueprint )
	{
		// Set blueprint settings.
		blueprint->SetNumGenes(kKnapsackItems);

		for(unsigned i=0;i<blueprint->GetNumGenes();i++)
			blueprint->SetGeneMinMax(i,Gene(0L),Gene(1L));

		// only boundary mutation
		blueprint->SetUniformMutate(true);
		blueprint->SetGaussianMutate(false);
		blueprint->SetBoundaryMutate(false);

		// only simple crossover
		blueprint->SetSimpleCrossover(true);
		blueprint->SetArithmeticCrossover(false);
		blueprint->SetHeuristicCrossover(false);

		// set mutation probability
		blueprint->SetMutationProb(0.05);

		// set crossover probability
		blueprint->SetCrossoverProb(0.9);


		blueprint->SetFitnessProc( KnapsackFitness,NULL);
	}
	return blueprint;
}



// print out size/values selected
void PrintKnapsack(const Genome *genome)
{
	cout << genome->GetFitness() << " -> ";

	for(unsigned i=0;i<kKnapsackItems;i++)
	{
		if( genome->GetGene(i).GetInteger() )
			cout << "(" << kSizes[i] << "," << kValues[i] << ") ";
	}

	cout << "\n";
}

// print out genes
void PrintKnapsackGenes(const Genome *genome)
{
	cout << genome->GetFitness() << " -> ";

	for(unsigned i=0;i<kKnapsackItems;i++)
	{
		cout << genome->GetGene(i).GetInteger() << " ";
	}

	cout << "\n";
}


void KnapsackTest(void)
{
	unsigned long gen = 0;
	GenomeBlueprint *blueprint = 0;
	Genome *genome = 0;
	Population *pop = 0;

	blueprint = KnapsackBlueprint();

	if( blueprint )
	{

		genome = new Genome(blueprint);
		

		if( genome )
			pop = new Population(genome);


		if( !pop )
			delete genome;
		else
		{
			pop->SetSize(20);
			pop->SetSurvivors(2); // keep a few good ones.
			pop->Seed(true); // start with zerovalued pop instead of random

			// works better with this set to true.
			pop->SetSelectByRank( true );

			double best = -1.0;

			for(;;)
			{
				gen++;

				if( (gen % 1000) == 0 )
					cout << "Gen: " << gen << "\n";

				unsigned index = pop->GetWinner();
				const Genome *g = pop->GetGenome(index);
				
				// print out fitness and genome
				if( g->GetFitness() > best )
				{
					cout << "Gen # " << gen << ": ";
					best = g->GetFitness();
					
					// Print out the genome
					PrintKnapsack( g );

					// exit condition
					if( best >= kBestKnapsackFitness )
						break;
				}

				pop->Evolve();
			}
			delete pop;
		}

		delete blueprint;
	}

}

// approximate a sin() curve from x=0 to x=PI/4 using an order-3 polynomial 
// at a total of 101 data points.
// 3 order polynomial means 4 coefficients.
const unsigned kPolyNumGenes = 4;

// max abs value of a coefficient.
const double kPolyMaxCo = 100.0;

const unsigned kPolyDataPoints = 101;

// no idea yet what this should be
const double kPolyMaxFitness = 10000.0;

// array of X values and array of Y values.
double gXValues[kPolyDataPoints], gYValues[kPolyDataPoints];

// fill in sin data points
static void PolyInit(void)
{
	for(unsigned i=0;i<kPolyDataPoints;i++)
	{
		double x,y;
		// was PI / 4.0
		x = PI / 2.0 * (double)i/(double)(kPolyDataPoints-1);
		// y = sin(x);
		y = cos(x);

		gXValues[i] = x;
		gYValues[i] = y;
	}
}

// return fitness of this specimen
static double PolyFitness( unsigned inNumGenes, const Gene *inGene, void *inFitnessData)
{
	// use mean squared error.
	double total = 0.0;

	for(unsigned i=0;i<kPolyDataPoints;i++)
	{
		// Could make this deal with different order polynomials.
		double x, y, x2, x3, err;
		x = gXValues[i];
		x2 = x*x;
		x3 = x2*x;

		y = inGene[0].GetReal()*x3;
		y += inGene[1].GetReal()*x2;
		y += inGene[2].GetReal()*x; // x
		y += inGene[3].GetReal(); // 1
		err = y - gYValues[i];
		total += err*err;
	}

	total /= (double)kPolyDataPoints;
	return (total == 0.0 ? kPolyMaxFitness : 1.0 / total); // fitness is less is better, so use the inverse.
}

GenomeBlueprint *PolyBlueprint(void)
{
	GenomeBlueprint *blueprint;

	// init arrays.
	PolyInit();

	
	blueprint = new GenomeBlueprint;

	if( blueprint )
	{
		// Set blueprint settings.
		blueprint->SetNumGenes(kPolyNumGenes);

		// set min and max
		for(unsigned i=0;i<blueprint->GetNumGenes();i++)
			blueprint->SetGeneMinMax(i,Gene(-kPolyMaxCo),Gene(kPolyMaxCo));

		// all kinds
		blueprint->SetUniformMutate(true);
		blueprint->SetGaussianMutate(true);
		blueprint->SetBoundaryMutate(true);

		// all kinds
		blueprint->SetSimpleCrossover(true);
		blueprint->SetArithmeticCrossover(true);
		blueprint->SetHeuristicCrossover(true);

		// set gaussian factor
		blueprint->SetGaussianFactor( 0.0 ); // makes it relative to current value.

		// set mutation probability
		blueprint->SetMutationProb(0.05);

		// set crossover probability
		blueprint->SetCrossoverProb(0.9);


		blueprint->SetFitnessProc( PolyFitness,NULL);

	}

	return blueprint;
}

// print out size/values selected
static void PrintPoly(const Genome *genome)
{
	cout << genome->GetFitness() << " -> ";

	for(unsigned i=0;i<kPolyNumGenes;i++)
	{
		double co = genome->GetGene(i).GetReal();
		if( co >= 0.0 )
			cout << "+" << co;
		else
			cout << co;
		cout << "*x^" << (kPolyNumGenes-i-1) << " ";
	}

	cout << "\n";
}

void PolyTest(void)
{
	unsigned long gen = 0;
	GenomeBlueprint *blueprint = 0;
	Genome *genome = 0;
	Population *pop = 0;

	blueprint = PolyBlueprint();

	if( blueprint )
	{

		genome = new Genome(blueprint);
		

		if( genome )
			pop = new Population(genome);


		if( !pop )
			delete genome;
		else
		{
			pop->SetSize(20);
			// runs much better with no survivors.
			pop->SetSurvivors(0); // keep a few good ones.
			pop->Seed();

			// works far better with this set to true.
			pop->SetSelectByRank( true );
			

			double best = -1.0;
			long bc = 0;
		
			for(;;)
			{
				gen++;

				if( (gen % 1000) == 0 )
					cout << "Gen: " << gen << "\n";

				unsigned index = pop->GetWinner();
				const Genome *g = pop->GetGenome(index);
				
				// print out fitness and genome
				if( g->GetFitness() > best )
				{
					bc++;
					if( (bc % 100 == 0) )
					{
						cout << "Gen # " << gen << ": ";
						best = g->GetFitness();
						
						// Print out the genome
						PrintPoly( g );

						// exit condition
						if( best >= kPolyMaxFitness )
							break;

				
					}
				}
				// max gen exit condition
				if( gen >= 100000 )
				{
					unsigned index = pop->GetWinner();
					const Genome *g = pop->GetGenome(index);
					// Print out the genome
					PrintPoly( g );

					break;
				}
				pop->Evolve();
			}
			delete pop;
		}

		delete blueprint;
	}

}

// blueprint as parameter, maxgen and bestfitness.
void MultiTest(GenomeBlueprint *blueprint, unsigned npops, unsigned popSize, unsigned popSurv, bool inZeroSeed, 
			   unsigned long inMaxGen, double inMaxFitness,
			   PrintProc printProc  )
{
	unsigned long gen = 0;
	Genome *genome = 0;
	MultiPopulation *multi = 0;
	Population *pop = 0;

	if( !blueprint )
		return;

	genome = new Genome(blueprint);


	if( genome )
		multi = new MultiPopulation;

	if( !multi )
		delete genome;
	else
	{
		while(npops--)
		{
			pop = new Population(genome->Copy());
			if(pop)
			{
				pop->SetSize(popSize);
				// runs much better with no survivors.
				pop->SetSurvivors(popSurv); // keep a few good ones.
				pop->Seed(inZeroSeed);

				// works far better with this set to true.
				pop->SetSelectByRank( true );

				multi->AddPopulation(pop);
			}
		}

		// free this now that we are done with it.
		delete genome; 

		double best = -1.0;
		long bc = 0;
	
		for(;;)
		{
			// gen++;
			gen += multi->GetNumPop() * multi->GetCycles();

			cout << "Gen: " << gen << "\n";

			const Genome *g = multi->GetWinner();
			
			// print out fitness and genome
			if( g->GetFitness() > best )
			{
				cout << "Gen # " << gen << ": ";
				best = g->GetFitness();
				
				// Print out the genome fitness
				printProc(g);
				

				// exit condition
				if( best >= inMaxFitness )
					break;
			
			}
			// max gen exit condition
			if( gen >= inMaxGen )
			{
				const Genome *g = multi->GetWinner();
				// Print out the genome fitness
				printProc(g);

				break;
			}
			multi->Evolve();

		}
		delete multi;
	}

}


// this function randomly generates knapsack problem data
void MakeKnapsackData(unsigned n, double avgSize, double avgValue)
{
	unsigned c;
	long size, value;
	std::cout << "---sizes---\n";
	c = n;
	while(c--)
	{
		size = (long)( GaussianRandom() * avgSize + avgSize + 0.5 );
		if( size <= 0 )
			size = 1; // can't have size < 1
		std::cout << size << ",\n";
	}
	std::cout << "---values---\n";
	c = n;
	while(c--)
	{
		value = (long)( GaussianRandom() * avgValue + avgValue + 0.5 );
		if( value <= 0 )
			value = 1; // can't have value < 1
		std::cout << value << ",\n";
	}
}

int main(int argc, char **argv)
{
	// this is utility code to generate knapsack problem test data.
//	MakeKnapsackData(100,20,5);
//	return 0;

	cout << "Welcome to Breed.\n";

	srand( time(0) );

	// Run some tests based on simple knapsack problem
	KnapsackTest();	// with single population
	MultiTest(KnapsackBlueprint(),3,20,2,true,100000,kBestKnapsackFitness,PrintKnapsack); // with multiple populations

	// Run some tests to fit a polynomial to a data set.
	PolyTest();
	MultiTest(PolyBlueprint(),3,20,0,false,100000,kPolyMaxFitness,PrintPoly);
	
	return 0;
}
