/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __GENOMEBLUEPRINT__
#include "genomeblueprint.h"
#endif


// This keeps up with number of, type, min, and max values
// for genomes in a central place, as well
// as what mutation and crossover operations are supported.
// also keeps up with fitness evaluation function.

GenomeBlueprint::GenomeBlueprint(void)
{
	mNumGenes = 0;
	mMin = 0;
	mMax = 0;

	mUniformMutate = mGaussianMutate = mBoundaryMutate = true;
	mSimpleCrossover = mArithmeticCrossover = mHeuristicCrossover = true;

	mGaussianFactor = 1.0;

	// changed the defaults from .01 and 0.7
	mMutationProb = 0.05;
	mCrossoverProb = 0.9;
}

GenomeBlueprint::~GenomeBlueprint()
{
	if( mMin )
	{
		delete[] mMin;
		mMin = 0;
	}
	if( mMax )
	{
		delete[] mMax;
		mMax = 0;
	}
}

//// Functions to setup values ////

// set number of genes, call this first because it zeroes them all
// returns false if fails.
bool GenomeBlueprint::SetNumGenes(unsigned inNumGenes)
{
	// free anything already around
	if( mMin )
	{
		delete[] mMin;
		mMin = 0;
	}
	if( mMax )
	{
		delete[] mMax;
		mMax = 0;
	}
	
	mNumGenes = inNumGenes;
	if( mNumGenes > 0 )
	{
		try
		{
			mMin = new Gene[ inNumGenes ];
			mMax = new Gene[ inNumGenes ];
		}
		catch(...)
		{
			return false;
		}
		if( !mMin || !mMax )
			return false;

		// set them all to real valued zeroes
		for(unsigned i=0;i<mNumGenes;i++)
			mMin[i] = mMax[i] = Gene(0.0);
	}

	return true;
}

// set min and max (and implicitly the type) of a gene
void GenomeBlueprint::SetGeneMinMax(unsigned index, const Gene& inMin, const Gene& inMax)
{
	mMin[index] = inMin;
	mMax[index] = inMax;
}


// returns # of different mutation operations
unsigned GenomeBlueprint::GetNumMutationTypes(void) const
{
	unsigned count = 0;

	if( mUniformMutate ) count++;
	if( mGaussianMutate ) count++;
	if( mBoundaryMutate ) count++;

	return count;
}

// returns # of different crossover operations
unsigned GenomeBlueprint::GetNumCrossoverTypes(void) const
{
	unsigned count = 0;

	if( mSimpleCrossover ) count++;
	if( mArithmeticCrossover ) count++;
	if( mHeuristicCrossover ) count++;

	return count;
}





