/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MATHUTILS__
#include "mathutils.h"
#endif



// Returns a normally distributed deviate with zero mean and unit variance.
double GaussianRandom(void)
{
	static int iset=0;
	static double gset;
	
	double fac,rsq,v1,v2;
	
	if( !iset )
	{ 
		// We dont have an extra deviate handy, so
		// pick two uniform numbers in the square extending
		// from -1 to +1 in each direction, 
		do {
			v1 = RandomN1To1();
			v2 = RandomN1To1();
			// see if they are in the unit circle,
			rsq = v1*v1 + v2*v2; 
			// and if they are not try again
		} while (rsq >= 1.0 || rsq == 0.0);

		fac = sqrt( -2.0 * log(rsq) / rsq );

		// Now make the Box-Muller transformation to get two normal deviates.
		// Return one and save the other for next time.
		gset = v1 * fac;
		iset = 1; // Set flag.
		
		return v2 * fac;
	}
	else
	{ 
		// We have an extra deviate handy
		iset = 0; // so unset the flag
		return gset; // and return it.
	}
}
