/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __POPULATION__
#define __POPULATION__

#ifndef __GENOME__
#include "genome.h"
#endif

class Population {
public:
	// we own the seed after this completes.
	Population( Genome *inSeed );
	virtual ~Population();

	// set the size of the population
	void SetSize( unsigned inSize );

	// set the number of survives kept unchanged each cycle, must be no greater than mSize
	void SetSurvivors( unsigned inSurvivors );

	// accessors
	unsigned GetSize(void) const { return mSize; }
	unsigned GetSurvivors(void) const { return mSurvivors; }
	const Genome *GetGenome(unsigned index) const { return mGenome[index]; }
	Genome *GetGenome(unsigned index) { return mGenome[index]; }

	// this is used by MultiPopulation, assumes inGenome->CalcFitness has already been called.
	Genome *SwapGenome(unsigned index, Genome *inGenome);

	// seed initial population, parameter is because some problems work much better with initial zero populations
	void Seed(bool inForceToZero=false);

	// evolve the population one generation forward
	void Evolve(void);

	// get index of winner
	unsigned GetWinner(void) const;

	void SetSelectByRank(bool inSelectByRank) { mSelectByRank = inSelectByRank; }
	bool GetSelectByRank(void) const { return mSelectByRank; }

protected:
	// sort population by fitness which must have already been calculated.
	void Sort(void);
	
	unsigned SelectParentByRank(void);
	unsigned SelectParentByFitness(void);

	// genomes will already have had their fitness values calculated.
	void SelectParents( Genome **outMother, Genome **outFather );

	// override this to change selection method, default is by rank,
	// return index of selected parent
	virtual unsigned SelectParent( void );

protected:
	bool mSelectByRank; // if true select by rank, if false select by fitness.
	unsigned mSize, mSurvivors;
	Genome *mSeed;
	Genome **mGenome; // array of genome ptrs
	Genome **mChildren; // temp storage for pointers to children.
};


#endif
